/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    i_ni488.h
 *
 * Purpose: Header for MX driver for National Instruments GPIB interfaces.
 */

#ifndef __I_NI488_H__
#define __I_NI488_H__

#include "mx_rec.h"

/* Define the data structure used by National Instruments GPIB interfaces. */

typedef struct {
	char interface_name[MXU_FILENAME_LENGTH + 1];
	int interface_address;
	int device_descriptor[MX_NUM_GPIB_ADDRESSES];
} MX_NI488;

/* Define all of the interface functions. */

MX_API mx_status_type mxi_ni488_initialize_type( long type );
MX_API mx_status_type mxi_ni488_create_record_structures( MX_RECORD *record );
MX_API mx_status_type mxi_ni488_finish_record_initialization(
							MX_RECORD *record );
MX_API mx_status_type mxi_ni488_delete_record( MX_RECORD *record );
MX_API mx_status_type mxi_ni488_print_interface_structure(
					FILE *file, MX_RECORD *record );
MX_API mx_status_type mxi_ni488_read_parms_from_hardware( MX_RECORD *record );
MX_API mx_status_type mxi_ni488_write_parms_to_hardware( MX_RECORD *record );
MX_API mx_status_type mxi_ni488_open( MX_RECORD *record );
MX_API mx_status_type mxi_ni488_close( MX_RECORD *record );

MX_API mx_status_type mxi_ni488_open_device( MX_GPIB *gpib, int address );

MX_API mx_status_type mxi_ni488_close_device( MX_GPIB *gpib, int address );

MX_API mx_status_type mxi_ni488_read_parms_from_device( MX_GPIB *gpib,
						int address );

MX_API mx_status_type mxi_ni488_write_parms_to_device( MX_GPIB *gpib,
						int address );

MX_API mx_status_type mxi_ni488_read( MX_GPIB *gpib,
						int address,
						char *buffer,
						size_t max_bytes_to_read,
						size_t *bytes_read,
						int flags);
MX_API mx_status_type mxi_ni488_write( MX_GPIB *gpib,
						int address,
						char *buffer,
						size_t bytes_to_write,
						size_t *bytes_written,
						int flags);

MX_API mx_status_type mxi_ni488_interface_clear( MX_GPIB *gpib );

MX_API mx_status_type mxi_ni488_device_clear( MX_GPIB *gpib );

MX_API mx_status_type mxi_ni488_selective_device_clear( MX_GPIB *gpib,
						int address );

MX_API mx_status_type mxi_ni488_local_lockout( MX_GPIB *gpib );

MX_API mx_status_type mxi_ni488_remote_enable( MX_GPIB *gpib, int address );

MX_API mx_status_type mxi_ni488_go_to_local( MX_GPIB *gpib, int address );

MX_API mx_status_type mxi_ni488_trigger( MX_GPIB *gpib, int address );

MX_API mx_status_type mxi_ni488_wait_for_service_request( MX_GPIB *gpib,
						double timeout );

MX_API mx_status_type mxi_ni488_serial_poll( MX_GPIB *gpib, int address,
					unsigned char *serial_poll_byte );

MX_API mx_status_type mxi_ni488_serial_poll_disable( MX_GPIB *gpib );

extern MX_RECORD_FUNCTION_LIST mxi_ni488_record_function_list;
extern MX_GPIB_FUNCTION_LIST mxi_ni488_gpib_function_list;

extern long mxi_ni488_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxi_ni488_rfield_def_ptr;

#define MXI_NI488_STANDARD_FIELDS \
  {-1, -1, "interface_name", MXFT_STRING, NULL, 1, {MXU_FILENAME_LENGTH}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_NI488, interface_name), \
	{sizeof(char)}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "interface_address", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_NI488, interface_address), \
	{0}, NULL, 0}, \
  \
  {-1, -1, "device_descriptor", MXFT_INT, NULL, 1, {MX_NUM_GPIB_ADDRESSES}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_NI488, device_descriptor), \
	{sizeof(int)}, NULL, 0}

#endif /* __I_NI488_H__ */

