/*
 * Name:    mx_mce.h
 *
 * Purpose: MX system header file for multichannel encoders.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2000 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __MX_MCE_H__
#define __MX_MCE_H__

#include "mx_rec.h"

typedef struct {
	MX_RECORD *record;
	long encoder_type;
	int overflow_set;
	int underflow_set;

	long maximum_num_values;

	long current_num_values;
	double *value_array;

	MX_RECORD *motor_record;
	double scale;
	double offset;
} MX_MCE;

#define MXLV_MCE_CURRENT_NUM_VALUES	1001
#define MXLV_MCE_VALUE_ARRAY		1002

#define MX_MCE_STANDARD_FIELDS \
  {-1, -1, "encoder_type", MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCE, encoder_type), \
	{0}, NULL, MXFF_READ_ONLY}, \
  \
  {-1, -1, "overflow_set", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCE, overflow_set), \
	{0}, NULL, MXFF_READ_ONLY}, \
  \
  {-1, -1, "underflow_set", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCE, underflow_set), \
	{0}, NULL, MXFF_READ_ONLY}, \
  \
  {-1, -1, "maximum_num_values", MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCE, maximum_num_values), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {MXLV_MCE_CURRENT_NUM_VALUES, -1, "current_num_values", \
				MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCE, current_num_values), \
	{0}, NULL, 0}, \
  \
  {MXLV_MCE_VALUE_ARRAY, -1, "value_array", MXFT_DOUBLE, \
		NULL, 1, {MXU_VARARGS_LENGTH}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCE, value_array), \
	{sizeof(double)}, NULL, MXFF_VARARGS }, \
  \
  {-1, -1, "motor_record", MXFT_RECORD, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCE, motor_record), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "scale", MXFT_DOUBLE, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCE, scale), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "offset", MXFT_DOUBLE, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCE, offset), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}


/* Encoder type */

#define MXT_MCE_ABSOLUTE_ENCODER	1
#define MXT_MCE_INCREMENTAL_ENCODER	2
#define MXT_MCE_DELTA_ENCODER		3

/*
 * The structure type MX_MCE_FUNCTION_LIST contains a list of pointers
 * to functions that vary from encoder type to encoder type.
 */

typedef struct {
	mx_status_type ( *get_overflow_status ) ( MX_MCE *encoder );
	mx_status_type ( *reset_overflow_status ) ( MX_MCE *encoder );
	mx_status_type ( *read ) ( MX_MCE *encoder );
	mx_status_type ( *get_current_num_values ) ( MX_MCE *encoder );
} MX_MCE_FUNCTION_LIST;

MX_API mx_status_type mx_mce_initialize_type(
			long record_type,
			long *num_record_fields,
			MX_RECORD_FIELD_DEFAULTS **record_field_defaults,
			long *maximum_num_values_varargs_cookie );

MX_API mx_status_type mx_mce_get_pointers( MX_RECORD *record,
					MX_MCE **encoder,
					MX_MCE_FUNCTION_LIST **fl_ptr,
					const char *calling_fname );

MX_API mx_status_type mx_mce_get_overflow_status( MX_RECORD *record,
				int *underflow_set, int *overflow_set );
MX_API mx_status_type mx_mce_reset_overflow_status( MX_RECORD *record );
MX_API mx_status_type mx_mce_read( MX_RECORD *record,
			unsigned long *num_values, double **value_array );
MX_API mx_status_type mx_mce_get_current_num_values( MX_RECORD *record,
						unsigned long *num_values );

#endif /* __MX_MCE_H__ */

